#!/usr/bin/env python
#BOILERPLATE###################################################################
#                                                                             #
#  This package wraps FCKeditor for use in the Zope web application server.   #
#  Copyright (C) 2005 Chad Whitacre < http://www.zetadev.com/ >               #
#                                                                             #
#  This library is free software; you can redistribute it and/or modify it    #
#  under the terms of the GNU Lesser General Public License as published by   #
#  the Free Software Foundation; either version 2.1 of the License, or (at    #
#  your option) any later version.                                            #
#                                                                             #
#  This library is distributed in the hope that it will be useful, but        #
#  WITHOUT ANY WARRANTY; without even the implied warranty of                 #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser    #
#  General Public License for more details.                                   #
#                                                                             #
#  You should have received a copy of the GNU Lesser General Public License   #
#  along with this library; if not, write to the Free Software Foundation,    #
#  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                #
#                                                                             #
#                                                                             #
###################################################################BOILERPLATE#

""" This script takes the FCKeditor base distribution in ../src/ and massages it
for use in Zope, outputting to ../skins/fckeditor/. Usage:

    $ ./base2zope.py
    $

"""

import os
import re
import shutil
import sys


##
# Initialize some variables.
##

PRODUCT_ROOT = os.path.realpath(os.path.join(sys.path[0],'..'))
SRC_ROOT     = os.path.join(PRODUCT_ROOT, '_src')
DEST_ROOT    = os.path.join(PRODUCT_ROOT, 'skins', 'fckeditor')
SRC_SKINS_ADDONS_ROOT = os.path.join(PRODUCT_ROOT, '_addons','skins')
DEST_SKINS_ADDONS_ROOT = os.path.join(PRODUCT_ROOT, 'skins','fckeditor','editor','skins')
SRC_PLUGINS_ADDONS_ROOT = os.path.join(PRODUCT_ROOT, '_addons','plugins')
DEST_PLUGINS_ADDONS_ROOT = os.path.join(PRODUCT_ROOT, 'skins','fckeditor','editor','plugins')


##
# Decide what to do if DEST_ROOT is already there.
##

def rm_rf(path):
    """ equivalent to rm -rf on Unix
    """
    if os.path.realpath(path) == '/':
        print 'will not rm -rf /' # better safe than sorry :-)
        sys.exit(1)
    else:
        for root, dirs, files in os.walk(path, topdown=False):
            for name in files:
                os.remove(os.path.join(root, name))
            for name in dirs:
                os.rmdir(os.path.join(root, name))

if os.path.exists(DEST_ROOT):
    force = sys.argv[1:2] == ['--force']
    if not force:
        answer = raw_input( "destination directory already exists; " +\
                            "delete and recreate? (y/n) [n] "
                            )
        force = answer.lower() == 'y'
    if force:   rm_rf(DEST_ROOT)
    else:       sys.exit(1)
else:
    os.makedirs(DEST_ROOT)





METADATA = """\
[default]
cache = FckHTTPCache"""

def metadata(filepath):
    """Given a filepath, write a complementary metadata file.
    """
    mdpath = '.'.join((filepath, 'metadata'))
    mdfile = file(mdpath,'w+')
    mdfile.write(METADATA)
    mdfile.close()

cache_me  = ('css','gif','html','js','xml')
ext_unwanted = ('asp','aspx','cfc','cfm','cgi','exe','htaccess','htc','php','pl','lasso','afp')
# files overloaded in fckeditor_plone layer
files_unwanted = ('fckconfig.js','fckeditor.py','fckstyles.xml.pt','fcktemplates.xml.pt')
# files changed (impossible to overload because of subdirectories in original skins)
files_changed = {'fck_link.html.pt':{'ResourceType':'File'},'fck_image.html.pt':{'ResourceType':'Image'},'fck_flash.html.pt':{'ResourceType':'Flash'}}


def makeSkinDirs(srcDir ,destDir):
  """
  Now walk the tree and transfer data to our output directory.
  """
  for path, dirs, files in os.walk(srcDir):
  
      # Determine and maybe create the destination.
      relpath = path[len(srcDir)+1:]
      destpath = os.path.join(destDir, relpath)
      if not os.path.exists(destpath):
          os.mkdir(destpath)
  
      for filename in files:
  
          src = os.path.join(path, filename)
  
          # Alter the output filename if necessary.
          ext = filename.split('.')[-1]
          if ext in ('html', 'xml'):
              filename += '.pt'
          dest = os.path.join(destpath, filename)
  
          # Create the new file if we want it.
          if (not filename.startswith('_')) and (ext not in ext_unwanted) and (filename not in files_unwanted):
  
              if files_changed.has_key(filename) :
  
                  # add Title + Description fields in upload forms
  
                  inputfile = file(src)
                  outputfile = file(dest, 'w+')
                  
                  resourceType = files_changed[filename]['ResourceType']
                  sizeLines = ''
                  if resourceType=='Image':
                    sizeLines='''
                                    			<span>Size</span>&nbsp;
                                    			<input type="radio" name="size" value="small" />small&nbsp;
                                    			<input type="radio" name="size" value="medium" checked />medium&nbsp;
                                    			<input type="radio" name="size" value="full"/>full&nbsp;<br />                    
                              '''
  
                  for line in inputfile.readlines():
  
                      if '<input id="txtUploadFile" ' in line :   
                         outputfile.write(line)
                         outputfile.write('''
                                    			<input type="hidden" name="Type" value="%s" />
                                          <span  fckLang="DlgGenTitle">Title</span><br />
                                    			<input id="Title" name="Title" style="WIDTH: 200px" type="text" /><br />%s
                                    			<span  fckLang="DlgDocMeDescr">Description</span><br />
                                    			<textarea id="Description" name="Description" rows="2" cols="30"></textarea>                       
                                          ''' %(resourceType, sizeLines))
                      else:
                         outputfile.write(line)
                          
                  inputfile.close()
                  outputfile.close()
  
              else:
                  shutil.copy(src, dest)
  
              if ext in cache_me:
                  metadata(dest)
  
      # skip svn directories
      if '.svn' in dirs:
          dirs.remove('.svn')
  
      # also skip extra FCKeditor directories
      for dirname in dirs[:]:
          if dirname.startswith('_'):
              dirs.remove(dirname)
          #filemanager is Zope specific (in fckeditorplone layer)   
          #directory removed 
          elif dirname == 'filemanager':
              dirs.remove(dirname)

#Add base skin directory
makeSkinDirs(SRC_ROOT ,DEST_ROOT)
# Add new skins and plugins
# temporary removed since new skins are not compatibles
# makeSkinDirs(SRC_SKINS_ADDONS_ROOT,DEST_SKINS_ADDONS_ROOT)
makeSkinDirs(SRC_PLUGINS_ADDONS_ROOT,DEST_PLUGINS_ADDONS_ROOT)
